/*
	Copyright (c) 2010 Christiaan Welvaart
	
	Permission is hereby granted, free of charge, to any person obtaining
	a copy of this software and associated documentation files (the
	"Software"), to deal in the Software without restriction, including
	without limitation the rights to use, copy, modify, merge, publish,
	distribute, sublicense, and/or sell copies of the Software, and to
	permit persons to whom the Software is furnished to do so, subject to
	the following conditions:

	The above copyright notice and this permission notice shall be included
	in all copies or substantial portions of the Software.

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
	EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
	MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
	IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
	CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
	TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
	SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

package net.time4t.yarc;

import net.time4t.yarc.Target;

import org.cybergarage.upnp.ControlPoint;
import org.cybergarage.upnp.Device;
import org.cybergarage.upnp.device.DeviceChangeListener;
import org.cybergarage.upnp.device.NotifyListener;
import org.cybergarage.upnp.ssdp.SSDPPacket;

import java.util.Iterator;
import java.util.Map;

public class Main implements DeviceChangeListener, NotifyListener
{
	private java.util.TreeMap<String, net.time4t.yarc.ui.Device>	devices;
	private java.util.Vector<ReceiverInfo>				deviceInfo;
	
	public static void main(String[] args)
	{
		new Main();
	}

		public Main()
	{
		// register all supported device models
		deviceInfo = new java.util.Vector<ReceiverInfo>();
		deviceInfo.add(new RXV2065Info());
		
		devices = new java.util.TreeMap<String, net.time4t.yarc.ui.Device>();
		org.cybergarage.util.Debug.on();
		ControlPoint	controlPoint = new ControlPoint();
		controlPoint.addDeviceChangeListener(this);
		controlPoint.addNotifyListener(this);
		controlPoint.start();
		controlPoint.search();
	}
	
	public void deviceAdded(Device device)
	{
		System.err.println("================ device ==================");
		System.err.println("manufacturer: " + device.getManufacturer());
		System.err.println("model name:   " + device.getModelName());
		System.err.println("model number: " + device.getModelNumber());
		System.err.println("url base:     " + device.getURLBase());
		System.err.println("device type:  " + device.getDeviceType());
		System.err.println("address:      " + device.getSSDPPacket().getRemoteInetAddress());
		System.err.println("present url:  " + device.getPresentationURL());
		System.err.println("UDN:          " + device.getUDN());
		
		ReceiverInfo	info;
		int		i;
		
		for (i = 0; i < deviceInfo.size(); ++i)
		{
			info = deviceInfo.elementAt(i);
			
			if (device.getManufacturer().equals(info.getManufacturer()) && device.getModelNumber().equals(info.getModelNumber()))
			{
				try
				{
					devices.put(device.getUDN(), new net.time4t.yarc.ui.Device(new Target(device), info, this));
				}
				catch(java.io.IOException e)
				{
				}
				catch(ClassNotFoundException e)
				{
				}
				catch(InstantiationException e)
				{
				}
				catch(IllegalAccessException e)
				{
				}
			}
		}
	}
	
	public void deviceRemoved(Device dev)
	{
	}
	
	public void deviceNotifyReceived(SSDPPacket packet)
	{
		int	index = packet.getUSN().indexOf("::");
		if (index >= 0)
		{
			String	udn = packet.getUSN().substring(0, index);
			net.time4t.yarc.ui.Device	device = devices.get(udn);
			if (device != null)
			{
				device.notify(packet.getContentString());
			}
		}
	}
	
	public void handleDeviceUIClose(net.time4t.yarc.ui.Device device)
	{
		Iterator<Map.Entry<String, net.time4t.yarc.ui.Device>>	iterator = devices.entrySet().iterator();
		Map.Entry<String, net.time4t.yarc.ui.Device>		entry;
		
		while (iterator.hasNext())
		{
			entry = iterator.next();
			if (entry.getValue() == device)
			{
				devices.remove(entry.getKey());
				break;
			}
		}
		
		device.dispose();
		
		if (devices.size() == 0)
		{
			System.exit(0);
		}
	}
}
