/*
	Copyright (c) 2010 Christiaan Welvaart
	
	Permission is hereby granted, free of charge, to any person obtaining
	a copy of this software and associated documentation files (the
	"Software"), to deal in the Software without restriction, including
	without limitation the rights to use, copy, modify, merge, publish,
	distribute, sublicense, and/or sell copies of the Software, and to
	permit persons to whom the Software is furnished to do so, subject to
	the following conditions:

	The above copyright notice and this permission notice shall be included
	in all copies or substantial portions of the Software.

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
	EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
	MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
	IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
	CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
	TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
	SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

package net.time4t.yarc;

import java.net.HttpURLConnection;
import java.net.URL;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.dom.DOMSource; 
import javax.xml.transform.OutputKeys;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import org.cybergarage.upnp.Device;
import org.w3c.dom.bootstrap.DOMImplementationRegistry;
import org.w3c.dom.Document;
import org.w3c.dom.DOMImplementation;
import org.w3c.dom.Element;

public class Target
{
	private URL			url;
	private DOMImplementation	dom;
	private Device			device;

	/**
		Creates a new target for receiver control commands
		
		@param address the Internet address of the target
	 */
		public Target(Device device)
			throws java.net.MalformedURLException, ClassNotFoundException, InstantiationException, IllegalAccessException
	{
		this.device = device;
		url = new URL("http:/" + device.getSSDPPacket().getRemoteInetAddress().toString() + "/YamahaRemoteControl/ctrl");
		// get an instance of the DOMImplementation registry
		DOMImplementationRegistry registry = DOMImplementationRegistry.newInstance();
		// get a DOM implementation the Level 3 XML module
		dom = registry.getDOMImplementation("XML 3.0");
		
	}
	
	/**
		Sends a GET command to the receiver
		
		@param key a dot separated "node" name
	*/
	public Document get(String key)
	{
		Document		result = null;
		try
		{
			HttpURLConnection	connection;
			Document		xml = dom.createDocument(null, "YAMAHA_AV", null);
			String[]		tags = key.split("\\.");
			Element			element;
			Element			new_element;
			int			i;

			element = xml.getDocumentElement();
			element.setAttribute("cmd", "GET");
			for (i = 0; i < tags.length; ++i)
			{
				new_element = xml.createElement(tags[i]);
				element.appendChild(new_element);
				element = new_element;
			}
			element.appendChild(xml.createTextNode("GetParam"));

			connection = (HttpURLConnection)url.openConnection();
			connection.setDoOutput(true);
			connection.setRequestMethod("POST");

			DOMSource domSource = new DOMSource(xml);
			StreamResult streamResult = new StreamResult(connection.getOutputStream());
			TransformerFactory tf = TransformerFactory.newInstance();
			Transformer serializer = tf.newTransformer();
			serializer.setOutputProperty(OutputKeys.ENCODING,"utf-8");
			serializer.setOutputProperty(OutputKeys.INDENT,"yes");
			serializer.transform(domSource, streamResult); 

			result = DocumentBuilderFactory.newInstance().newDocumentBuilder().parse(connection.getInputStream());

			connection.connect();
		}
		catch (java.io.IOException e) {}
		catch (javax.xml.transform.TransformerConfigurationException e) {}
		catch (javax.xml.transform.TransformerException e) {}
		catch (javax.xml.parsers.ParserConfigurationException e) {}
		catch (org.xml.sax.SAXException e) {}
		
		return result;
	}
	
	public void put(String key, Properties value)
	{
		try
		{
			HttpURLConnection	connection;
			Document		xml = dom.createDocument(null, "YAMAHA_AV", null);
			String[]		tags = key.split("\\.");
			Element			element;
			Element			new_element;
			int			i;

			element = xml.getDocumentElement();
			element.setAttribute("cmd", "PUT");
			for (i = 0; i < tags.length; ++i)
			{
				new_element = xml.createElement(tags[i]);
				element.appendChild(new_element);
				element = new_element;
			}
			
			for (i = 0; i < value.size(); ++i)
			{
				new_element = xml.createElement(value.getKey(i));
				element.appendChild(new_element);
				new_element.appendChild(xml.createTextNode(value.getValue(i)));
			}

			connection = (HttpURLConnection)url.openConnection();
			connection.setDoOutput(true);
			connection.setRequestMethod("POST");

			DOMSource domSource = new DOMSource(xml);
			StreamResult streamResult = new StreamResult(connection.getOutputStream());
			TransformerFactory tf = TransformerFactory.newInstance();
			Transformer serializer = tf.newTransformer();
			serializer.setOutputProperty(OutputKeys.ENCODING,"utf-8");
			serializer.setOutputProperty(OutputKeys.INDENT,"yes");
			serializer.transform(domSource, streamResult); 

			DocumentBuilderFactory.newInstance().newDocumentBuilder().parse(connection.getInputStream());

			connection.connect();
		}
		catch (java.io.IOException e) {}
		catch (javax.xml.transform.TransformerConfigurationException e) {}
		catch (javax.xml.transform.TransformerException e) {}
		catch (javax.xml.parsers.ParserConfigurationException e) {}
		catch (org.xml.sax.SAXException e) {}
	}
	
	public String getModelName()
	{
		return device.getModelName();
	}
	
}
