/*
	Copyright (c) 2010 Christiaan Welvaart
	
	Permission is hereby granted, free of charge, to any person obtaining
	a copy of this software and associated documentation files (the
	"Software"), to deal in the Software without restriction, including
	without limitation the rights to use, copy, modify, merge, publish,
	distribute, sublicense, and/or sell copies of the Software, and to
	permit persons to whom the Software is furnished to do so, subject to
	the following conditions:

	The above copyright notice and this permission notice shall be included
	in all copies or substantial portions of the Software.

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
	EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
	MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
	IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
	CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
	TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
	SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

package net.time4t.yarc.ui;

import javax.swing.BoxLayout;
import java.util.Arrays;
import java.util.TreeSet;
import java.util.Vector;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.xml.parsers.DocumentBuilderFactory;
import org.w3c.dom.Document;
import org.w3c.dom.NodeList;

import net.time4t.yarc.Target;

public class Device implements java.awt.event.WindowListener
{
	private JFrame			window;
	private Zone[]			zones;
	private	Vector<String>		inputNames;
	private Vector<InputPanel>	inputs;
	private Vector<Vector<Boolean>>	inputStatus;
	private java.util.List<String>	zoneNames;
	private Target			target;
	private net.time4t.yarc.Main	owner;

		public Device(Target target, net.time4t.yarc.ReceiverInfo info, net.time4t.yarc.Main owner)
	{
		JPanel		zonesPanel;
		JPanel		inputsPanel;
		int		i, j;
		Document	xml;
		NodeList	nodes;
		BoxLayout	window_layout;
		BoxLayout	layout;
		InputPanel	input;
		Vector<Boolean>	inputZones;

		this.target = target;
		this.owner = owner;
		inputNames = new Vector<String>(Arrays.asList(info.getZones()[0].getInputs()));
		// check for optional inputs - assume first zone supports all inputs
		for (i = 0; i < info.getZones()[0].getOptionalInputs().length; ++i)
		{
			xml = target.get(info.getZones()[0].getOptionalInputs()[i] + ".Config");
			nodes = xml.getElementsByTagName("Device");
			if (nodes.getLength() == 0)
			{
				continue;
			}
			if (nodes.item(0).getTextContent().equals("Not Connected") || nodes.item(0).getTextContent().equals("Ready"))
			{
				inputNames.add(info.getZones()[0].getOptionalInputs()[i]);
			}
		}
		inputStatus = new Vector<Vector<Boolean>>();

		// window contains zones & inputs
		window = new JFrame("YARC: " + target.getModelName());
		window_layout = new BoxLayout(window.getContentPane(), BoxLayout.Y_AXIS);
		window.getContentPane().setLayout(window_layout);
		
		// zones panel
		zonesPanel = new JPanel();
		layout = new BoxLayout(zonesPanel, BoxLayout.Y_AXIS);
		zonesPanel.setLayout(layout);
		window.getContentPane().add(zonesPanel);

		zones = new Zone[info.getZones().length];
		zoneNames = new java.util.ArrayList<String>();
		for (i = 0; i < 3; ++i)
		{
			zones[i] = new Zone(info.getZones()[i], this);
			zonesPanel.add(zones[i].getPanel());
			zoneNames.add(info.getZones()[i].getName());
		}
		
		inputsPanel = new JPanel();
		layout = new BoxLayout(inputsPanel, BoxLayout.Y_AXIS);
		inputsPanel.setLayout(layout);
		window.getContentPane().add(inputsPanel);
				
		inputs = new Vector<InputPanel>();
		for (i = 0; i < inputNames.size(); ++i)
		{
			input = InputPanelFactory.newInputPanel(inputNames.elementAt(i), this);
			inputs.add(input);
			inputZones = new Vector<Boolean>();
			inputStatus.add(inputZones);
			for (j = 0; j < zones.length; ++j)
			{
				inputZones.add(new Boolean(false));
			}
			if (input != null)
			{
				inputsPanel.add(input);
			}
//			input.setVisible(false);
		}
		
		window.pack();
		window.addWindowListener(this);
		window.setVisible(true);
		
		update();
		
		net.time4t.yarc.Properties	properties = new net.time4t.yarc.Properties();
		properties.put("Notice", "On");
		target.put("System.Event", properties);
	}
	
	public void dispose()
	{
		net.time4t.yarc.Properties	properties = new net.time4t.yarc.Properties();
		properties.put("Notice", "Off");
		target.put("System.Event", properties);

		window.dispose();
	}

	private void update()
	{
		int	i;
		
		for (i = 0; i < zones.length; ++i)
		{
			zones[i].update();
		}
		
		updateInputs();
	}
	
	private void updateInputs()
	{
		int	i, j;

		for (i = 0; i < inputs.size(); ++i)
		{
			boolean	enabled = false;
	
			if (inputs.elementAt(i) != null)
			{
				for (j = 0; j < zones.length; ++j)
				{
					enabled |= inputStatus.elementAt(i).elementAt(j).booleanValue();
				}
				
				inputs.elementAt(i).updateStatus(enabled);
			}
		}
	}
	
	public void notify(String data)
	{
		Document	xml = null;
		NodeList	mainNodes;
		NodeList	nodes;
		NodeList	propertyNodes;
		String		zone;
		Vector<String>	properties;
		int		i;
		int		zoneNum;
		TreeSet<String>	dirtyInputs = new TreeSet<String>();

		if (data.length() > 0)
		{
			try
			{
				xml = DocumentBuilderFactory.newInstance().newDocumentBuilder().parse(new org.xml.sax.InputSource(new java.io.CharArrayReader(data.toCharArray(), 0, data.length() - 1)));
			}
			catch (java.io.IOException e) {}
			catch (javax.xml.parsers.ParserConfigurationException e) {}
			catch (org.xml.sax.SAXException e) {}

			if ((xml != null) && xml.getDocumentElement().getNodeName().equals("YAMAHA_AV"))
			{
				properties = new Vector<String>();
				
				mainNodes = xml.getDocumentElement().getChildNodes();
				for (zoneNum = 0; zoneNum < mainNodes.getLength(); ++zoneNum)
				{
					zone = mainNodes.item(zoneNum).getNodeName();
					nodes = mainNodes.item(zoneNum).getChildNodes();
					
					for (i = 0; i < nodes.getLength(); ++i)
					{
						if (!nodes.item(i).getNodeName().equals("Property"))
						{
							System.err.println("net.time4t.yarc.ui.Main.notify(): property node name is not Property !");
						}
						propertyNodes = nodes.item(i).getChildNodes();
						if (propertyNodes.getLength() > 1)
						{
							System.err.println("net.time4t.yarc.ui.Main.notify(): lvl 3 nodes length > 1 !");
						}
						if (propertyNodes.item(0).getTextContent().equals("Play_Info"))
						{
							dirtyInputs.add(zones[zoneNames.indexOf(zone)].getInputName());
						}
						else
						{
							properties.add(propertyNodes.item(0).getTextContent());
						}
					}

					if (properties.size() > 0)
					{
						zones[zoneNames.indexOf(zone)].notify(properties);
					}
				}
				
				// iterate over dirty inputs and update only those inputs' panels
				for (i = 0; i < inputNames.size(); ++i)
				{
					if (dirtyInputs.contains(inputNames.elementAt(i)))
					{
						if (inputs.elementAt(i) != null)
						{
							boolean	enabled = false;
							int	j;
							
							for (j = 0; j < zones.length; ++j)
							{
								enabled |= inputStatus.elementAt(i).elementAt(j).booleanValue();
							}
				
							inputs.elementAt(i).updateStatus(enabled);
						}
					}
				}
			}
			else
			{
				System.err.println("net.time4t.yarc.ui.Main.notify(): top node is not YAMAHA_AV !");
			}
		}
	}
	
	void notifyZoneInput(String zoneName, String inputName)
	{
		int	i;
		int	j;
		Boolean	value;
		boolean	oldEnabled = false;
		boolean	newEnabled = false;
		
		for (i = 0; i < inputNames.size(); ++i)
		{
			value = Boolean.FALSE;
			oldEnabled = false;
			newEnabled = false;
			
			if (inputNames.elementAt(i).equals(inputName))
			{
				value = Boolean.TRUE;
			}

			if (inputs.elementAt(i) != null)
			{
				for (j = 0; j < zones.length; ++j)
				{
					oldEnabled |= inputStatus.elementAt(i).elementAt(j).booleanValue();
				}
			}

			inputStatus.elementAt(i).set(zoneNames.indexOf(zoneName), value);

			if (inputs.elementAt(i) != null)
			{
				for (j = 0; j < zones.length; ++j)
				{
					newEnabled |= inputStatus.elementAt(i).elementAt(j).booleanValue();
				}
				
				if (oldEnabled != newEnabled)
				{
					inputs.elementAt(i).updateStatus(newEnabled);
				}
			}
		}
	}
	
	boolean inputAvailable(String name)
	{
		return inputNames.indexOf(name) >= 0;		
	}
	

	public Document get(String key)
	{
		return target.get(key);
	}

	public void put(String key, net.time4t.yarc.Properties value)
	{
		target.put(key, value);
	}
	
	
	public void windowActivated(java.awt.event.WindowEvent e)
	{
	}
	
	public void windowClosed(java.awt.event.WindowEvent e)
	{
	}
	
	public void windowClosing(java.awt.event.WindowEvent e)
	{
		owner.handleDeviceUIClose(this);
	}
	
	public void windowDeactivated(java.awt.event.WindowEvent e)
	{
	}
	
	public void windowDeiconified(java.awt.event.WindowEvent e)
	{
	}
	
	public void windowIconified(java.awt.event.WindowEvent e)
	{
	}
	
	public void windowOpened(java.awt.event.WindowEvent e)
	{
	}
	
}
