/*
	Copyright (c) 2010 Christiaan Welvaart
	
	Permission is hereby granted, free of charge, to any person obtaining
	a copy of this software and associated documentation files (the
	"Software"), to deal in the Software without restriction, including
	without limitation the rights to use, copy, modify, merge, publish,
	distribute, sublicense, and/or sell copies of the Software, and to
	permit persons to whom the Software is furnished to do so, subject to
	the following conditions:

	The above copyright notice and this permission notice shall be included
	in all copies or substantial portions of the Software.

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
	EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
	MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
	IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
	CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
	TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
	SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

package net.time4t.yarc.ui;

import java.awt.Color;
import javax.swing.border.LineBorder;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;

import org.w3c.dom.Document;
import org.w3c.dom.NodeList;

class PCPanel extends InputPanel implements java.awt.event.ActionListener
{
	static final long serialVersionUID = 2956256856363971964L;
	
	private final String[]	repeat_names = { "Off", "Single", "All" };

	private Device		device;
	private boolean		updating;

	private JButton		stop;
	private JButton		play;
	private JButton		backward;
	private JButton		forward;
	private JCheckBox	shuffle;
	private JComboBox	repeat;
	
	private	JLabel		artist;
	private JLabel		album;
	private JLabel		song;

		PCPanel(Device device)
	{
		ImageIcon	icon;
		java.net.URL	imageUrl = null;
		JLabel		label;
		BoxLayout	panelLayout;
		JPanel		configPanel;
		BoxLayout	configLayout;
		JPanel		topPanel;
		BoxLayout	topLayout;
		JPanel		artistPanel;
		BoxLayout	artistLayout;
		JPanel		albumPanel;
		BoxLayout	albumLayout;
		JPanel		songPanel;
		BoxLayout	songLayout;

		this.device = device;
		updating = false;
		
		panelLayout = new BoxLayout(this, BoxLayout.Y_AXIS);
		this.setLayout(panelLayout);
		label = new JLabel("PC");
		label.setAlignmentX(java.awt.Component.LEFT_ALIGNMENT);
		this.add(label);
		
		configPanel = new JPanel();
		configLayout = new BoxLayout(configPanel, BoxLayout.Y_AXIS);
		configPanel.setLayout(configLayout);
		configPanel.setBorder(new LineBorder(Color.BLACK));
		this.add(configPanel);

		artistPanel = new JPanel();
		artistLayout = new BoxLayout(artistPanel, BoxLayout.X_AXIS);
		artistPanel.setLayout(artistLayout);
		label = new JLabel("artist: ");
		artist = new JLabel("");
		artistPanel.add(label);
		artistPanel.add(artist);
		artistPanel.add(javax.swing.Box.createHorizontalGlue());
		configPanel.add(artistPanel);
		
		albumPanel = new JPanel();
		albumLayout = new BoxLayout(albumPanel, BoxLayout.X_AXIS);
		albumPanel.setLayout(albumLayout);
		label = new JLabel("album: ");
		album = new JLabel("");
		albumPanel.add(label);
		albumPanel.add(album);
		albumPanel.add(javax.swing.Box.createHorizontalGlue());
		configPanel.add(albumPanel);
		
		songPanel = new JPanel();
		songLayout = new BoxLayout(songPanel, BoxLayout.X_AXIS);
		songPanel.setLayout(songLayout);
		label = new JLabel("song: ");
		song = new JLabel("");
		songPanel.add(label);
		songPanel.add(song);
		songPanel.add(javax.swing.Box.createHorizontalGlue());
		configPanel.add(songPanel);

		topPanel = new JPanel();
		topLayout = new BoxLayout(topPanel, BoxLayout.X_AXIS);
		topPanel.setLayout(topLayout);
		
		imageUrl = TunerPanel.class.getResource("/images/stop.gif");
		icon = new ImageIcon(imageUrl);
		stop = new JButton(icon);
		stop.setMargin(new java.awt.Insets(2, 2, 2, 2));
		imageUrl = TunerPanel.class.getResource("/images/right.gif");
		icon = new ImageIcon(imageUrl);
		play = new JButton(icon);
		play.setMargin(new java.awt.Insets(2, 2, 2, 2));
		imageUrl = TunerPanel.class.getResource("/images/left2.gif");
		icon = new ImageIcon(imageUrl);
		backward = new JButton(icon);
		backward.setMargin(new java.awt.Insets(2, 2, 2, 2));
		imageUrl = TunerPanel.class.getResource("/images/right2.gif");
		icon = new ImageIcon(imageUrl);
		forward = new JButton(icon);
		forward.setMargin(new java.awt.Insets(2, 2, 2, 2));
		
		topPanel.add(stop);
		topPanel.add(play);
		topPanel.add(backward);
		topPanel.add(forward);
		topPanel.add(javax.swing.Box.createHorizontalGlue());

		label = new JLabel("repeat");
		repeat = new JComboBox(repeat_names);
		shuffle = new JCheckBox("shuffle");

		topPanel.add(label);
		topPanel.add(repeat);
		topPanel.add(shuffle);

		configPanel.add(topPanel);
		
		stop.addActionListener(this);
		play.addActionListener(this);
		backward.addActionListener(this);
		forward.addActionListener(this);
		shuffle.addActionListener(this);
		repeat.addActionListener(this);
	}

	void updateStatus(boolean enabled)
	{
		Document	xml;
		NodeList	nodes;
		boolean		playing = false;

		forward.setEnabled(enabled);
		backward.setEnabled(enabled);
		shuffle.setEnabled(enabled);
		repeat.setEnabled(enabled);

		xml = device.get("PC.Play_Info");
		if (xml == null)
		{
			return;
		}
		nodes = xml.getElementsByTagName("Artist");
		if (nodes.getLength() > 0)
		{
			artist.setText(nodes.item(0).getTextContent());
		}
		nodes = xml.getElementsByTagName("Album");
		if (nodes.getLength() > 0)
		{
			album.setText(nodes.item(0).getTextContent());
		}
		nodes = xml.getElementsByTagName("Song");
		if (nodes.getLength() > 0)
		{
			song.setText(nodes.item(0).getTextContent());
		}
		nodes = xml.getElementsByTagName("Shuffle");
		if (nodes.getLength() > 0)
		{
			shuffle.setSelected(nodes.item(0).getTextContent().equals("On"));
		}
		nodes = xml.getElementsByTagName("Repeat");
		if (nodes.getLength() > 0)
		{
			int		selectedIndex;
			int		newIndex;
			
			selectedIndex = repeat.getSelectedIndex();
			newIndex = java.util.Arrays.asList(repeat_names).indexOf(nodes.item(0).getTextContent());
			if (selectedIndex != newIndex)
			{
				repeat.setSelectedIndex(newIndex);
			}
		}
		nodes = xml.getElementsByTagName("Status");
		if (nodes.getLength() > 0)
		{
			playing = nodes.item(0).getTextContent().equals("Play");
		}
		stop.setEnabled(enabled && playing);
		play.setEnabled(enabled && !playing);
	}

	public void actionPerformed(java.awt.event.ActionEvent e)
	{
		if (!updating)
		{
			if (e.getSource() == stop)
			{
				net.time4t.yarc.Properties	properties = new net.time4t.yarc.Properties();
				properties.put("Play", "Stop");
				device.put("PC.Play_Control", properties);
			}
			else if (e.getSource() == play)
			{
				net.time4t.yarc.Properties	properties = new net.time4t.yarc.Properties();
				properties.put("Play", "Play");
				device.put("PC.Play_Control", properties);
			}
			else if (e.getSource() == backward)
			{
				net.time4t.yarc.Properties	properties = new net.time4t.yarc.Properties();
				properties.put("Skip", "Rev");
				device.put("PC.Play_Control", properties);
			}
			else if (e.getSource() == forward)
			{
				net.time4t.yarc.Properties	properties = new net.time4t.yarc.Properties();
				properties.put("Skip", "Fwd");
				device.put("PC.Play_Control", properties);
			}
			else if (e.getSource() == shuffle)
			{
				net.time4t.yarc.Properties	properties = new net.time4t.yarc.Properties();
				properties.put("Shuffle", shuffle.isSelected() ? "On" : "Off");
				device.put("PC.Play_Control.Play_Mode", properties);
			}
			else if (e.getSource() == repeat)
			{
				net.time4t.yarc.Properties	properties = new net.time4t.yarc.Properties();
				properties.put("Repeat", repeat.getSelectedItem().toString());
				device.put("PC.Play_Control.Play_Mode", properties);
			}
		}
	}
}
