/*
	Copyright (c) 2010 Christiaan Welvaart
	
	Permission is hereby granted, free of charge, to any person obtaining
	a copy of this software and associated documentation files (the
	"Software"), to deal in the Software without restriction, including
	without limitation the rights to use, copy, modify, merge, publish,
	distribute, sublicense, and/or sell copies of the Software, and to
	permit persons to whom the Software is furnished to do so, subject to
	the following conditions:

	The above copyright notice and this permission notice shall be included
	in all copies or substantial portions of the Software.

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
	EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
	MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
	IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
	CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
	TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
	SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

package net.time4t.yarc.ui;

import java.awt.Color;
import java.util.Arrays;
import java.util.Vector;
import javax.swing.border.LineBorder;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import net.time4t.yarc.ZoneInfo;
import org.w3c.dom.Document;
import org.w3c.dom.NodeList;

public class Zone implements java.awt.event.ActionListener
{
	private String	name;
	private Vector<String>	inputNames;
	private String[]	programNames;
	private Device		device;
	private boolean		DSP;
	
	private	JCheckBox	enable;
	private	JComboBox	input;
	private JCheckBox	mute;
	private JButton		volUp;
	private JButton		volDown;
	private JButton		volUp2;
	private JButton		volDown2;
	private JTextField	volume;
	private JComboBox	program;

	private JPanel		panel;
	private	JPanel		controlPanel;
	private	JPanel		programPanel;
	
	private	boolean		updating;
	
		public Zone(ZoneInfo info, Device device)
	{
		BoxLayout	panelLayout;
		BoxLayout	controlLayout;
		BoxLayout	muteLayout;
		BoxLayout	volumeLayout;
		BoxLayout	volumeCenterLayout;
		BoxLayout	volumeRightLayout;
		BoxLayout	inputLayout;
		BoxLayout	inputGlueLayout;
		BoxLayout	muteGlueLayout;
		BoxLayout	inputMuteLayout;
		BoxLayout	inputMuteProgramLayout;
		BoxLayout	programLayout;
		JPanel		enablePanel;
		JPanel		inputMutePanel;
		JPanel		inputMuteProgramPanel;
		JPanel		inputPanel;
		JPanel		inputGluePanel;
		JPanel		muteGluePanel;
		JPanel		mutePanel;
		JPanel		volumePanel;
		JPanel		volumeCenterPanel;
		JPanel		volumeRightPanel;
		JLabel		label;
		JLabel		label2;
		ImageIcon	icon;
		int		i;
		
		this.name = info.getName();
		inputNames = new Vector<String>(Arrays.asList(info.getInputs()));
		for (i = 0; i < info.getOptionalInputs().length; ++i)
		{
			if (device.inputAvailable(info.getOptionalInputs()[i]))
			{
				inputNames.add(info.getOptionalInputs()[i]);
			}
		}
		programNames = info.getPrograms();
		this.device = device;
		updating = false;
		detect_dsp();
		
		panel = new JPanel();
		panelLayout = new BoxLayout(panel, BoxLayout.Y_AXIS);
		panel.setLayout(panelLayout);

		enablePanel = new JPanel();
		enablePanel.add(new JLabel(name));
		enable = new JCheckBox();
		enablePanel.add(enable);
		enable.setAlignmentX(java.awt.Component.LEFT_ALIGNMENT);
		enablePanel.setAlignmentX(java.awt.Component.RIGHT_ALIGNMENT);

		controlPanel = new JPanel();
		controlPanel.setBorder(new LineBorder(Color.BLACK));

		panel.add(enablePanel);
		panel.add(controlPanel);

		// main zone control panel
		controlLayout = new BoxLayout(controlPanel, BoxLayout.X_AXIS);
		controlPanel.setLayout(controlLayout);
		
		// input + mute + program panel
		if (DSP)
		{
			inputMuteProgramPanel = new JPanel();
			inputMuteProgramLayout = new BoxLayout(inputMuteProgramPanel, BoxLayout.Y_AXIS);
			inputMuteProgramPanel.setLayout(inputMuteProgramLayout);
		}
		else
		{
			inputMuteProgramPanel = null;
		}
		
		// input + mute panel
		inputMutePanel = new JPanel();
		inputMuteLayout = new BoxLayout(inputMutePanel, BoxLayout.X_AXIS);
		inputMutePanel.setLayout(inputMuteLayout);
		(DSP ? inputMuteProgramPanel : controlPanel).add(inputMutePanel);
		
		// input panel		
		inputPanel = new JPanel();
		inputLayout = new BoxLayout(inputPanel, BoxLayout.X_AXIS);
		inputPanel.setLayout(inputLayout);
		inputPanel.add(new JLabel("Input"));
		inputGluePanel = new JPanel();
		inputGlueLayout = new BoxLayout(inputGluePanel, BoxLayout.Y_AXIS);
		inputGluePanel.setLayout(inputGlueLayout);
		inputGluePanel.add(javax.swing.Box.createVerticalGlue());
		input = new JComboBox(inputNames);
		inputGluePanel.add(input);
		inputGluePanel.add(javax.swing.Box.createVerticalGlue());
		inputPanel.add(inputGluePanel);
//		inputPanel.setAlignmentX(java.awt.Component.LEFT_ALIGNMENT);
		inputMutePanel.add(inputPanel);
		
		inputMutePanel.add(javax.swing.Box.createHorizontalGlue());
		
		muteGluePanel = new JPanel();
		muteGlueLayout = new BoxLayout(muteGluePanel, BoxLayout.Y_AXIS);
		muteGluePanel.setLayout(muteGlueLayout);
		muteGluePanel.add(javax.swing.Box.createVerticalGlue());
		mutePanel = new JPanel();
		muteLayout = new BoxLayout(mutePanel, BoxLayout.X_AXIS);
		mutePanel.setLayout(muteLayout);
		mutePanel.add(new JLabel("Mute"));
		mute = new JCheckBox();
		mutePanel.add(mute);
//		mutePanel.setAlignmentX(java.awt.Component.RIGHT_ALIGNMENT);
		muteGluePanel.add(mutePanel);
		muteGluePanel.add(javax.swing.Box.createVerticalGlue());
		inputMutePanel.add(muteGluePanel);
		
		volumePanel = new JPanel();
		volumeLayout = new BoxLayout(volumePanel, BoxLayout.X_AXIS);
		volumePanel.setLayout(volumeLayout);
		
		volumeCenterPanel = new JPanel();
		volumeCenterLayout = new BoxLayout(volumeCenterPanel, BoxLayout.Y_AXIS);
		volumeCenterPanel.setLayout(volumeCenterLayout);
		
		volumeRightPanel = new JPanel();
		volumeRightLayout = new BoxLayout(volumeRightPanel, BoxLayout.Y_AXIS);
		volumeRightPanel.setLayout(volumeRightLayout);
		volumeRightPanel.setAlignmentX(java.awt.Component.RIGHT_ALIGNMENT);

		label = new JLabel("Volume");
		java.net.URL	imageUrl = null;
		imageUrl = Zone.class.getResource("/images/up.gif");
		icon = new ImageIcon(imageUrl);
		volUp = new JButton(icon);
		volUp.setMargin(new java.awt.Insets(-4,6,-4,6));
		volume = new JTextField();
		volume.setEditable(false);
		volume.setColumns(5);
		imageUrl = Zone.class.getResource("/images/down.gif");
		icon = new ImageIcon(imageUrl);
		volDown = new JButton(icon);
		volDown.setMargin(new java.awt.Insets(-4,6,-4,6));
		imageUrl = Zone.class.getResource("/images/up2.gif");
		icon = new ImageIcon(imageUrl);
		volUp2 = new JButton(icon);
		volUp2.setMargin(new java.awt.Insets(-3,6,-3,6));
		label2 = new JLabel("dB");
		imageUrl = Zone.class.getResource("/images/down2.gif");
		icon = new ImageIcon(imageUrl);
		volDown2 = new JButton(icon);
		volDown2.setMargin(new java.awt.Insets(-3,6,-3,6));
		volumePanel.add(label);
		volumeCenterPanel.add(volUp);
		volumeCenterPanel.add(volume);
		volumeCenterPanel.add(volDown);
		volumePanel.add(volumeCenterPanel);
		volumeRightPanel.add(volUp2);
		volumeRightPanel.add(label2);
		volumeRightPanel.add(volDown2);
		volumePanel.add(volumeRightPanel);

		if (DSP && (programNames != null))
		{
			programPanel = new JPanel();
			programLayout = new BoxLayout(programPanel, BoxLayout.X_AXIS);
			programPanel.setLayout(programLayout);
			programPanel.add(new JLabel("Program"));
			program = new JComboBox(programNames);
			programPanel.add(program);
			inputMuteProgramPanel.add(programPanel);
			inputMuteProgramPanel.add(javax.swing.Box.createVerticalGlue());
			controlPanel.add(inputMuteProgramPanel);

			program.addActionListener(this);
		}
		else
		{
			program = null;
		}
		controlPanel.add(javax.swing.Box.createHorizontalGlue());
		controlPanel.add(volumePanel);
		
		enable.addActionListener(this);
		input.addActionListener(this);
		mute.addActionListener(this);
		volUp.addActionListener(this);
		volDown.addActionListener(this);
		volUp2.addActionListener(this);
		volDown2.addActionListener(this);
	}
	
	private void detect_dsp()
	{
		Document	xml;
		NodeList	nodes;
		
		xml = device.get(name + ".Basic_Status");
		nodes = xml.getElementsByTagName("Pgm_Sel");
		DSP =  (nodes.getLength() > 0);
	}
	
	public JPanel getPanel()
	{
		return panel;
	}

	public void update()
	{
		Document	xml;
		NodeList	nodes;
		float		val;
		boolean		enabled = false;
		
		updating = true;
		
		xml = device.get(name + ".Basic_Status");
		nodes = xml.getElementsByTagName("Power");
		if (nodes.getLength() > 0)
		{
			enabled = nodes.item(0).getTextContent().equals("On");
		}
		enable.setSelected(enabled);
		input.setEnabled(enabled);
		mute.setEnabled(enabled);
		if (DSP)
		{
			program.setEnabled(enabled);
		}
		volUp.setEnabled(enabled);
		volDown.setEnabled(enabled);
		volUp2.setEnabled(enabled);
		volDown2.setEnabled(enabled);
		nodes = xml.getElementsByTagName("Input_Sel");
		if (nodes.getLength() > 0)
		{
			input.setSelectedIndex(inputNames.indexOf(nodes.item(0).getTextContent()));
			device.notifyZoneInput(name, enabled ? nodes.item(0).getTextContent() : "");
		}
		if (DSP && (programNames != null))
		{
			nodes = xml.getElementsByTagName("Pgm_Sel");
			if (nodes.getLength() > 0)
			{
				program.setSelectedIndex(java.util.Arrays.asList(programNames).indexOf(nodes.item(0).getTextContent()));
			}
		}
		nodes = xml.getElementsByTagName("Mute");
		if (nodes.getLength() > 0)
		{
			mute.setSelected(nodes.item(0).getTextContent().equals("On"));
		}
		nodes = xml.getElementsByTagName("Val");
		if (nodes.getLength() > 0)
		{
			val = Integer.parseInt(nodes.item(0).getTextContent());
			nodes = xml.getElementsByTagName("Exp");
			if (nodes.getLength() > 0)
			{
				val /= Math.pow(10.0, Integer.parseInt(nodes.item(0).getTextContent()));
				volume.setText(Float.toString(val));
			}
		}
		
		updating = false;
	}
	
	public void notify(Vector<String> properties)
	{
/*
		int	i;
		for (i = 0; i < properties.size(); ++i)
		{
			System.out.println("net.time4t.yarc.ui.Zone.notify(): " + name + " property=" + properties.elementAt(i));
		}
*/
		update();
	}
	
	public void actionPerformed(java.awt.event.ActionEvent e)
	{
		if (!updating)
		{
			if (e.getSource() == enable)
			{
				net.time4t.yarc.Properties	properties = new net.time4t.yarc.Properties();
				properties.put("Power", enable.isSelected() ? "On" : "Standby" );
				device.put(name + ".Power_Control", properties);
			}
			else if ((e.getSource() == input) && (e.getActionCommand().equals("comboBoxChanged")))
			{
				net.time4t.yarc.Properties	properties = new net.time4t.yarc.Properties();
				properties.put("Input_Sel", input.getSelectedItem().toString());
				device.put(name + ".Input", properties);
			}
			else if (e.getSource() == mute)
			{
				net.time4t.yarc.Properties	properties = new net.time4t.yarc.Properties();
				properties.put("Mute", mute.isSelected() ? "On" : "Off" );
				device.put(name + ".Vol", properties);
			}
			else if (DSP && (e.getSource() == program) && (e.getActionCommand().equals("comboBoxChanged")))
			{
				net.time4t.yarc.Properties	properties = new net.time4t.yarc.Properties();
				properties.put("Pgm_Sel", program.getSelectedItem().toString());
				device.put(name + ".Surr", properties);
			}
			else if ((e.getSource() == volUp) || (e.getSource() == volDown))
			{
				net.time4t.yarc.Properties	properties = new net.time4t.yarc.Properties();
				properties.put("Up_Down", (e.getSource() == volUp) ? "Up" : "Down");
				device.put(name + ".Vol", properties);
			}
			else if ((e.getSource() == volUp2) || (e.getSource() == volDown2))
			{
				float	val = Float.parseFloat(volume.getText());
				val += (e.getSource() == volUp2) ? 4.0 : -4.0;
				net.time4t.yarc.Properties	properties = new net.time4t.yarc.Properties();
				properties.put("Val", Integer.toString((int)(val * 10)));
				properties.put("Exp", "1");
				properties.put("Unit", "dB");
				device.put(name + ".Vol.Lvl", properties);
			}
		}
	}
	
	public String getInputName()
	{
		return input.getSelectedItem().toString();
	}
}
